//
//  Copyright (C) 1998 Matthias Hoelzer
//  email:  hoelzer@physik.uni-wuerzburg.de
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the7 implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//


#include <fcntl.h>
#include <unistd.h>
#include <stream.h>

#include <qlist.h>
#include <qfile.h>
#include <qdstream.h>
#include <qregexp.h>
#include <kmsgbox.h>
#include <kapp.h>
#include <ktopwidget.h>

#include "widgets.h"


QString title;
bool separateOutput = FALSE;

int callServer(QStrList args) 
{
  int result;
  QString msg;
  char buffer[255];

  // check if called with the right parameters
  if (strcmp(args.at(1), "--kid") != 0 || args.count()<3)
  {
    cerr << args.at(0) << kapp->i18n(": wrong parameters") << endl;
    return -1;
  }

  // open the argument pipe
  QFile ap(args.at(2));
  if (!ap.open(IO_WriteOnly))
  {
    cerr << args.at(0) << kapp->i18n(": could not open argument pipe ") << args.at(2) << endl;
    return -1;
  }
  QDataStream as(&ap);

  // pipe the arguments to the server
  as << args;
  ap.flush();

  // open the return pipe
  QFile rp(args.at(3));
  if (!rp.open(IO_ReadWrite))
  {
    cerr << args.at(0) << kapp->i18n(": could not open return pipe ") << args.at(3) << endl;
    return -1;
  }
  QDataStream rs(&rp);

  // do not wait for termination of server
  if (strcmp(args.at(4),"--kexit") == 0)
    exit(0);

  // process the return values
  while (1) {

    // wait for the return arguments
    rs >> result;

    // process the return code
    switch (result) {

      // return code
      case 0: rs >> result;
              rp.close();
              ap.close();
              exit(result);
              break;

      // print value on stderr
      case 1: rs >> msg;
	      fprintf(stderr, "%s\n", msg.data());
              break;

      // feed gauge with stdin
      case 2: while (cin.getline(buffer, 255)) {
                args.clear();
                args.append("--gaugearg");
                args.append(buffer);
                as << args; 
                ap.flush();
              }
              ap.close();
              exit(0);
              break;
    }  
  }
}


void menu(QString text, QString width, QString /*height*/, QString menuheight, QStrList args, int index)
{
  QString result;
  int w = convert(width, 40);
  int mh = convert(menuheight, 10);

  int retcode = menuBox(0, w, mh, title, text, args, index, result);
  cerr << result << endl;
  exit((retcode == QDialog::Accepted) ? 0 : -1);
}


void checklist(QString text, QString width, QString /* height */, QString menuheight, QStrList args, int index)
{
  QStrList result;
  unsigned int i;
  int w = convert(width, 40);
  int mh = convert(menuheight, 10);

  int retcode = checkList(0, w, mh, title, text, args, index, separateOutput, result);

  for (i=0; i<result.count(); i++)
    cerr << result.at(i) << endl;;	    
  exit((retcode == QDialog::Accepted) ? 0 : -1);
}


void radiobox(QString text, QString width, QString /* height */, QString menuheight, QStrList args, int index)
{
  QString result;
  int w = convert(width, 40);
  int mh = convert(menuheight, 10);

  int retcode = radioBox(0, w, mh, title, text, args, index, result);
  cerr << result << endl; 
  exit((retcode == QDialog::Accepted) ? 0 : -1);
}


int directCommand(QStrList args)
{
  int index, count=args.count();
  char *arg;

  for (index=0; index < count; index++) {

    arg = args.at(index);

    // --title text
    if ((strcmp(arg,"--title") == 0) && (index+1<=count)) 
    {
      title = args.at(index+1);
      index++;
      continue;
    }

    // --separate-output
    if ((strcmp(arg,"--separate-output") == 0)) 
    {
      separateOutput = TRUE;
      continue;
    }

    // --clear
    if ((strcmp(arg,"--clear") == 0)) 
    {
      continue;
    }

    // --create-rc
    if ((strcmp(arg,"--create-rc") == 0)) 
    {
      continue;
    }

    // --yesno text width height
    if ((strcmp(arg,"--yesno") == 0) && (index+1<count)) 
    {
      if (KMsgBox::yesNo(0, title, args.at(index+1), 0, kapp->i18n("Yes"), kapp->i18n("No")) == 1)
        return 0;
      else
        return -1;
    }

    // --msgbox text width height
    if ((strcmp(arg,"--msgbox") == 0) && (index+1<count)) {
      KMsgBox::message(0, title, args.at(index+1),0,"Ok");
      return 0;
    }

    // --infobox text width height
    if ((strcmp(arg,"--infobox") == 0) && (index+1<count)) {
      return 0;
    }

    // --inputbox text width height init
    if ((strcmp(arg,"--inputbox") == 0) && (index+4<count)) 
    {
      QString result;
      int w = convert(args.at(index+2), 40);

      int retcode = inputBox(0, w, title, args.at(index+1), args.at(index+4), result);
      cerr << result << endl;
      return (retcode == QDialog::Accepted) ? 0 : -1;
    }

    // --textbox file width height
    if ((strcmp(arg,"--textbox") == 0) && (index+3<count)) 
    {
      int w = convert(args.at(2), 40);
      int h = convert(args.at(3), 15);

      textBox(0, w, h, title, args.at(index+1));
      return 0;
    }

    // --menu text width height menuheight [tag item]
    if ((strcmp(arg,"--menu") == 0) && (index+6<count)) {
      menu(args.at(index+1),args.at(index+2),args.at(index+3),
           args.at(index+4),args,index+5);
      return 0;
    }

    // --checklist text width height menuheight [tag item status]
    if ((strcmp(arg,"--checklist") == 0) && (index+7<count)) {
      checklist(args.at(index+1),args.at(index+2),args.at(index+3),
           args.at(index+4),args,index+5);
      return 0;
    }

    // --radiolist text width height menuheight [tag item status]
    if ((strcmp(arg,"--radiolist") == 0) && (index+7<count)) {
      radiobox(args.at(index+1),args.at(index+2),args.at(index+3),
           args.at(index+4),args,index+5);
      return 0;
    }
  }
}


int main(int argc, char *argv[]) 
{
  KApplication app(argc, argv);

  QString argument;
  QStrList args;
  int pos;

  // create list of arguments
  for (int arg=0; arg<argc; arg++) 
  {
    argument = argv[arg];
  
    while ((pos = argument.find("\\n")) >= 0) 
    {
      argument.replace(pos, 2, "\n");
    }

    args.append(argument);
  }

  // option "--kid" -> call the server
  if (strcmp(args.at(1), "--kid") == 0)
  {
    return callServer(args);
  }

  // execute direct kdialog command
  return directCommand(args);
}
